@@ -28,7 +28,7 @@
 #endif
 
 #define APPNAME "QCPUFreq"
-#define APPVERSION "0.3.3"
+#define APPVERSION "0.3.4"
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -39,8 +39,6 @@ MainWindow::MainWindow(QWidget *parent) :
     helperProcess( this ),
     //create a new, stackable help window
     helpWindow( this ),
-    //set minFreq to 0
-    minFreq(0),
     //create UI refresh timer
     refreshTimer( this ),
     //create a QGraphicsScene for the little chip icon
@@ -55,6 +53,7 @@ MainWindow::MainWindow(QWidget *parent) :
 
     ui->setupUi(this);
 
+    init();
     refresh();
 
     // enable auto rotation
@@ -81,8 +80,9 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(&refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
     connect(ui->actionOverclocking, SIGNAL(toggled(bool)), this, SLOT(setOverclocking()));
 
+
     //disable overclocking button on vanilla kernels
-    if ( getScalingFreq(0) <= 600000 ) {
+    if (!powerKernel) {
         ui->actionOverclocking->setDisabled(true);
     }
 
@@ -110,6 +110,12 @@ void MainWindow::about()
 void MainWindow::adjustFreq()
 {
     int newmax = getScalingFreq( ui->freq_adjust->sliderPosition() );
+
+    if (newmax == getMaxFreq() ) {
+        //we do not need to change anything in this case
+        return;
+    }
+
     QString max;
 
     //maxfreq should not be smaller than minfreq, because we do not want to decrease minfreq
@@ -215,32 +221,7 @@ int MainWindow::getMaxFreq()
   */
 int MainWindow::getMinFreq()
 {
-    if (this->minFreq == 0) {
-        QString min = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_min_freq" );
-        //check if avoid file exists (only on power kernel)
-        QFile file( "/sys/devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
-        if (file.exists()) {
-            QString avoid = readSysFile( "devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
-            QStringList avoidList = avoid.split( " " );
-
-            //check if min is in avoid_frequencies
-            for (int i = getScalingStep( min.toInt() ); i>0; --i) {
-                if (!avoidList.contains(min.setNum( getScalingFreq(i) ))) {
-                    this->minFreq = min.toInt();
-                    return this->minFreq;
-                }
-            }
-
-            //should not happen at all
-            this->minFreq = 125000;
-            return this->minFreq;
-        } else {
-            this->minFreq = min.toInt();
-            return this->minFreq;
-        }
-    } else {
-        return this->minFreq;
-    }
+    return this->minFreq;
 }
 
 
@@ -249,23 +230,20 @@ int MainWindow::getMinFreq()
   */
 int MainWindow::getScalingFreq(int step)
 {
-    QString tmp = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies" );
-    QStringList freqs = tmp.split( " " );
     step = step - 1;
     if ( step < 0 )
          step = 0;
-    if ( step > getScalingSteps() )
-        step = getScalingSteps();
+    if ( step > getScalingSteps() - 1 )
+        step = getScalingSteps() - 1;
 
-    tmp = freqs[ step ];
-    return tmp.toInt();
+    return this->scalingFrequencies[ step ].toInt();
 }
 
 
 /**
   * Returns the name of the current CPU frequency scaling governor
   *
-  * \return	name of governor
+  * @return	QString - name of governor
   */
 QString MainWindow::getScalingGovernor()
 {
@@ -275,26 +253,25 @@ QString MainWindow::getScalingGovernor()
 
 /**
   * Returns the amount of available scaling steps.
+  *
+  * @return int
   */
 int MainWindow::getScalingSteps()
 {
-    QString tmp = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies" );
-    QStringList freqs = tmp.split( " " );
-    return (freqs.size() - 1);
+    return this->scalingSteps;
 }
 
 
 /**
   * Returns the scaling step for the specified frequency.
+  *
+  * @return int
   */
 int MainWindow::getScalingStep( int freq )
 {
-    for( int i = 1; i <= getScalingSteps(); ++i ) {
-           if ( getScalingFreq(i) == freq )
-                return i;
-    }
-
-    return 1;
+    QString tmp;
+    tmp.setNum(freq);
+    return this->scalingFrequencies.indexOf(tmp) + 1;
 }
 
 
@@ -309,10 +286,11 @@ int MainWindow::getSmartReflexState()
 #if defined(Q_WS_MAEMO_5)
     QString tmp = readSysFile( "power/sr_vdd1_autocomp" );
 
-    if ( tmp == "1" )
-	return 1;
-    else
-	return 0;
+    if ( tmp == "1" ) {
+        return 1;
+    } else {
+        return 0;
+    }
 #else
     //disable UI checkbox
     ui->sr_box->setDisabled( true );
@@ -322,6 +300,54 @@ int MainWindow::getSmartReflexState()
 }
 
 
+/**
+  * Initializes internal variables, such as:
+  *  - scalingSteps
+  *  - scalingFrequencies
+  *  - minFreq
+  *  - powerKernel
+  */
+void MainWindow::init()
+{
+    this->minFreq = 0;
+    QString freqs = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies" );
+    QStringList freqList = freqs.split( " " );
+    //change the order of the QStringList - last element becomes first
+    for (int i=freqList.size() - 1; i>=0; --i) {
+        if (freqList.at(i) != "")
+            this->scalingFrequencies << freqList.at(i);
+    }
+    this->scalingSteps = (this->scalingFrequencies.size());
+
+    //set minFreq and check avoid_frequencies
+    QString min = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_min_freq" );
+    //check if avoid file exists (only on power kernel)
+    QFile file( "/sys/devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
+    if (file.exists()) {
+        QString avoid = readSysFile( "devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
+        QStringList avoidList = avoid.split( " " );
+
+        //check if min is in avoid_frequencies
+        for (int i = getScalingStep( min.toInt() ); i <= this->scalingSteps; ++i) {
+            min.setNum( getScalingFreq(i) );
+            if (!avoidList.contains(min)) {
+                this->minFreq = min.toInt();
+                break;
+            }
+        }
+    } else {
+        this->minFreq = min.toInt();
+    }
+
+    //check if we are using a power kernel
+    if ( getScalingFreq(getScalingSteps()) > 600000 ) {
+        this->powerKernel = true;
+    } else {
+        this->powerKernel = false;
+    }
+}
+
+
 /**
   * Reads any file in /sys/
   *
@@ -375,15 +401,14 @@ void MainWindow::refresh()
 
     //smart reflex button
     if ( getSmartReflexState() == 1 )
-	ui->sr_box->setCheckState( Qt::Checked );
+        ui->sr_box->setCheckState( Qt::Checked );
     else
-	ui->sr_box->setCheckState( Qt::Unchecked );
+        ui->sr_box->setCheckState( Qt::Unchecked );
 
 
     //display frequency slider
     ui->freq_adjust->setMinimum( 1 );
     ui->freq_adjust->setMaximum( getScalingSteps() );
-    ui->freq_adjust->setInvertedAppearance( true );
     ui->freq_adjust->setSliderPosition( getScalingStep(getMaxFreq()) );
 
     //ui->retranslateUi(this);
@@ -399,22 +424,22 @@ void MainWindow::orientationChanged()
 
     //check whether we are using portrait or landscape mode
     if ( usePortrait() ) {
-	//in portrait mode we want to display the large image
-	image.load( ":/img/chip256" );
-	scene.clear();
-	scene.addPixmap(  image  );
+        //in portrait mode we want to display the large image
+        image.load( ":/img/chip256" );
+        scene.clear();
+        scene.addPixmap(  image  );
 
-	ui->graphicsPortrait->setScene( &scene );
-	ui->graphicsPortrait->setMaximumSize( 256, 256 );
-	ui->graphicsLandscape->setMaximumSize( 0, 0 );
+        ui->graphicsPortrait->setScene( &scene );
+        ui->graphicsPortrait->setMaximumSize( 256, 256 );
+        ui->graphicsLandscape->setMaximumSize( 0, 0 );
     } else {
-	image.load( ":/img/chip128" );
-	scene.clear();
-	scene.addPixmap(  image  );
+        image.load( ":/img/chip128" );
+        scene.clear();
+        scene.addPixmap(  image  );
 
-	ui->graphicsLandscape->setScene( &scene );
-	ui->graphicsLandscape->setMaximumSize( 128, 128 );
-	ui->graphicsPortrait->setMaximumSize( 0, 0 );
+        ui->graphicsLandscape->setScene( &scene );
+        ui->graphicsLandscape->setMaximumSize( 128, 128 );
+        ui->graphicsPortrait->setMaximumSize( 0, 0 );
     }
 }
 
@@ -456,7 +481,7 @@ void MainWindow::setSmartReflex()
     if ( getSmartReflexState() == 1 )
         callHelper( "set_sr", "off");
     else {
-        QMaemo5InformationBox::information(this, tr( "SmartReflex support is known to be unstable on some devices and may cause random reboots." ), QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr( "SmartReflex support is known to be unstable on some devices and may cause random reboots." ), 0);
         callHelper( "set_sr", "on");
     }
 
