@@ -23,6 +23,7 @@
 #include <QGraphicsScene>
 #include <QTimer>
 #include <QProcess>
+#include <QStringList>
 
 #include "helpwindow.h"
 
@@ -65,10 +66,18 @@ private:
     QProcess helperProcess;
     //! The help window
     HelpWindow helpWindow;
+    //! Initializes the application
+    void init();
     int minFreq;
+    //! Are we using a power kernel?
+    bool powerKernel;
     QString readSysFile( QString sys_file );
     //! the timer for refreshing the UI
     QTimer refreshTimer;
+    //! The amount of scaling steps
+    int scalingSteps;
+    //! The available scaling frequencies
+    QStringList scalingFrequencies;
     //! the QGraphicsScene will contain the large chip icon displayed in the UI
     QGraphicsScene scene;
     bool showSudoError;
